"""
This plugin searches for GitLab tokens
"""
import re

from detect_secrets.plugins.base import RegexBasedDetector


class GitLabTokenDetector(RegexBasedDetector):
    """Scans for GitLab tokens."""

    secret_type = 'GitLab Token'

    denylist = [
        # ref:
        #  - https://docs.gitlab.com/ee/security/token_overview.html#gitlab-tokens
        #  - https://gitlab.com/groups/gitlab-org/-/epics/8923
        #  - https://github.com/gitlabhq/gitlabhq/blob/master/gems
        #        /gitlab-secret_detection/lib/gitleaks.toml#L6-L76

        # `gl..-` prefix and a token of length >20
        #    characters are typically alphanumeric, underscore, dash
        # Most tokens are generated either with:
        #  - `Devise.friendly_token`, a string with a default length of 20, or
        #  - `SecureRandom.hex`, default data size of 16 bytes, encoded in different ways.
        # String length may vary depending on the type of token, and probably
        # even GL-settings in the future, so we expect between 20 and 50 chars.

        # Personal Access Token - glpat
        # Deploy Token - gldt
        # Feed Token - glft
        # OAuth Access Token - glsoat
        # Runner Token - glrt
        re.compile(
            r'(glpat|gldt|glft|glsoat|glrt)-'
            r'[A-Za-z0-9_\-]{20,50}(?!\w)',
        ),

        # Runner Registration Token
        re.compile(r'GR1348941[A-Za-z0-9_\-]{20,50}(?!\w)'),

        # CI/CD Token - `glcbt` or `glcbt-XY_` where XY is a 2-char hex 'partition_id'
        re.compile(r'glcbt-([0-9a-fA-F]{2}_)?[A-Za-z0-9_\-]{20,50}(?!\w)'),

        # Incoming Mail Token - generated by SecureRandom.hex, default length 16 bytes
        # resulting token length is 26 when Base-36 encoded
        re.compile(r'glimt-[A-Za-z0-9_\-]{25}(?!\w)'),

        # Trigger Token - generated by `SecureRandom.hex(20)`
        re.compile(r'glptt-[A-Za-z0-9_\-]{40}(?!\w)'),

        # Agent Token - generated by `Devise.friendly_token(50)`
        # tokens have a minimum length of 50 chars, up to 1024 chars
        re.compile(r'glagent-[A-Za-z0-9_\-]{50,1024}(?!\w)'),

        # GitLab OAuth Application Secret - generated by `SecureRandom.hex(32)`
        # -> becomes 64 base64-encoded characters
        re.compile(r'gloas-[A-Za-z0-9_\-]{64}(?!\w)'),
    ]



"""
This plugin searches for PyPI tokens
"""
import re

from detect_secrets.plugins.base import RegexBasedDetector


class PypiTokenDetector(RegexBasedDetector):
    """Scans for PyPI tokens."""
    secret_type = 'PyPI Token'

    denylist = [
        # refs https://warehouse.pypa.io/development/token-scanning.html
        # pypi.org token
        re.compile(r'pypi-AgEIcHlwaS5vcmc[A-Za-z0-9-_]{70,}'),

        # test.pypi.org token
        re.compile(r'pypi-AgENdGVzdC5weXBpLm9yZw[A-Za-z0-9-_]{70,}'),
    ]



"""
This plugin searches for Telegram bot tokens
"""
import re

import requests

from ..constants import VerifiedResult
from detect_secrets.plugins.base import RegexBasedDetector


class TelegramBotTokenDetector(RegexBasedDetector):
    """Scans for Telegram bot tokens."""
    secret_type = 'Telegram Bot Token'

    denylist = [
        # refs https://core.telegram.org/bots/api#authorizing-your-bot
        re.compile(r'^\d{8,10}:[0-9A-Za-z_-]{35}$'),
    ]

    def verify(self, secret: str) -> VerifiedResult:  # pragma: no cover
        response = requests.get(
            'https://api.telegram.org/bot{}/getMe'.format(
                secret,
            ),
        )
        return (
            VerifiedResult.VERIFIED_TRUE
            if response.status_code == 200
            else VerifiedResult.VERIFIED_FALSE
        )



import re

from .base import RegexBasedDetector


class IPPublicDetector(RegexBasedDetector):
    """Scans for public ip address (ipv4)

    Some non-public ipv4 addresses are ignored, such as:
        - 127.
        - 10.
        - 172.(16-31)
        - 192.168.
        - 169.254. - Link Local Address IPv4

    Reference:
    https://www.iana.org/assignments/ipv4-address-space/ipv4-address-space.xhtml
    https://en.wikipedia.org/wiki/Private_network
    """
    secret_type = 'Public IP (ipv4)'

    denylist_ipv4_address = r"""
        (?<![\w.])         # Negative lookbehind: Ensures no preceding word character or dot
        (                  # Start of the main capturing group
            (?!            # Negative lookahead: Ensures the following pattern doesn't match
                192\.168\. # Exclude "192.168."
                |127\.     # Exclude "127."
                |10\.      # Exclude "10."
                |169\.254\. # Exclude IPv4 Link Local Address (169.254.0.0/16)
                |172\.(?:1[6-9]|2[0-9]|3[01])   # Exclude "172." with specific ranges
            )
            (?:            # Non-capturing group for octets
                           # Match numbers 0-255 followed by dot, properly handle leading zeros
                (?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])\.
            ){3}           # Repeat for three octets
                           # Match final octet (0-255), properly handle leading zeros
            (?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])
            (?:            # Optional non-capturing group for port number
                :\d{1,5}   # Match colon followed by 1 to 5 digits
            )?
        )                  # End of the main capturing group
        (?![\w.])          # Negative lookahead: Ensures no following word character or dot
    """

    denylist = [
        re.compile(denylist_ipv4_address, flags=re.IGNORECASE | re.VERBOSE),
    ]



"""
This plugin searches for OpenAI tokens
"""
import re

from detect_secrets.plugins.base import RegexBasedDetector


class OpenAIDetector(RegexBasedDetector):
    """Scans for OpenAI tokens."""
    secret_type = 'OpenAI Token'

    denylist = [
        # refs https://community.openai.com/t/what-are-the-valid-characters-for-the-apikey/288643
        # User api keys (legacy): 'sk-[20 alnum]T3BlbkFJ[20 alnum]'
        # Project-based api keys: 'sk-[project-name]-[20 alnum]T3BlbkFJ[20 alnum]'
        re.compile(r'sk-[A-Za-z0-9-_]*[A-Za-z0-9]{20}T3BlbkFJ[A-Za-z0-9]{20}'),
    ]
